@@ -110,10 +110,11 @@ sub generate_cover_db {
             next;
         }
 
-        my $hits              = $data->{ $original_filename };
-        my ($statement, $md5) = process_file_structure($file, $digests);
+        my $hits       = $data->{ $original_filename };
+        my $statements = [ sort { $a <=> $b } keys %$hits ];
+        my $md5        = process_file_structure($file, $statements, $digests);
 
-        $run->{count}{$file}{statement} = [ map +( $hits->{ $_ } // 0), @{ $statement } ];
+        $run->{count}{$file}{statement} = [ @{$hits}{@$statements}  ];
         $run->{digests}{$file}          = $md5;
     }
 
@@ -127,41 +128,28 @@ sub generate_cover_db {
 }
 
 sub process_file_structure {
-    my ($file, $digests) = @_;
+    my ($file, $statements, $digests) = @_;
 
     my $content = read_file($file, { binmode => ':raw'});
     my $md5     = md5_hex( $content );
-    my $statement;
 
     if (! exists($digests->{ $md5 })) {
         $digests->{ $md5 } = $file;
-        $statement         = write_structure($file, $md5, $content);
-    } else {
-        my $structure_data = read_file( "$STRUCTURE/$md5", { binmode => ':raw' });
-        my $structure      = coverdb_decode($structure_data);
-        $statement         = $structure->{statement};
+        write_structure($file, $md5, $statements);
     }
-    return $statement, $md5;
+    return $md5;
 }
 
 sub write_structure {
-    my ($file, $md5, $content) = @_;
-
-    my @lines   = split /\n/, $content;
-    my $statement = [
-        map  $_->[0],                                   # get the line number
-        grep $_->[1] !~ /^\s*#/ && $_->[1] !~ /^\s*$/,  # ignore comments and whitespaces
-        map  [ $_ + 1, $lines[$_] ], 0 .. $#lines,      # enumerate($data)
-    ];
+    my ($file, $md5, $statements) = @_;
 
     my $structure = {
         file       => $file,
         digest     => $md5,
         start      => {},
-        statement  => $statement,
+        statement  => $statements,
         subroutine => [],
     };
 
     write_file( "$STRUCTURE/$md5", { binmode => ':raw' }, coverdb_encode( $structure ));
-    return $statement;
 }
