@@ -46,11 +46,17 @@ CoverList* cover_create(void);
  */
 void cover_destroy(CoverList* cover);
 
+/*
+ * Add an executed file:line to the CoverList; will create CoverNode
+ * for file, if it doesn't already exist.
+ */
+void cover_add_covered(CoverList* cover, const char* file, int line);
+
 /*
  * Add a file:line to the CoverList; will create CoverNode for file, if it
  * doesn't already exist.
  */
-CoverNode* cover_add(CoverList* cover, const char* file, int line);
+void cover_add_line(CoverList* cover, const char* file, int line);
 
 /*
  * Dump all data to a given file stream.
